#include <propeller.h>
#include "ws2812.h"

#ifndef TRUE
#define TRUE    1
#define FALSE   0
#endif

// driver header structure
typedef struct {
    uint32_t    jmp_inst;
    uint32_t    resettix;
    uint32_t    bit0hi;
    uint32_t    bit0lo;
    uint32_t    bit1hi;
    uint32_t    bit1lo;
    uint32_t    swaprg;
} ws2812_hdr;

int ws2812_init(ws2812_t *state)
{
    return ws_init(state, 50, 350, 800, 700, 600, TYPE_GRB);
}

int ws2812b_init(ws2812_t *state)
{
    return ws_init(state, 50, 350, 900, 900, 350, TYPE_GRB);
}

// -- usreset - is reset timing (us)
// -- ns0h is 0-bit high timing (ns)
// -- ns0l is 0-bit low timing (ns)
// -- ns1h is 1-bit high timing (ns)
// -- ns1l is 1-bit low timing (ns)
// -- type is TYPE_GRB for ws2812 or ws2812b
int ws_init(ws2812_t *state, int usreset, int ns0h, int ns0l, int ns1h, int ns1l, int type)
{
    extern uint32_t binary_ws2812_driver_dat_start[];
    ws2812_hdr *hdr = (ws2812_hdr *)binary_ws2812_driver_dat_start;
    uint32_t ustix;

    ustix = CLKFREQ / 1000000;          // ticks in 1us

    hdr->resettix = ustix * usreset;
    hdr->bit0hi   = ustix * ns0h / 1000;
    hdr->bit0lo   = ustix * ns0l / 1000;
    hdr->bit1hi   = ustix * ns1h / 1000;
    hdr->bit1lo   = ustix * ns1l / 1000;
    hdr->swaprg   = (type == TYPE_GRB);
    
    state->command = 0;
    state->cog = cognew(hdr, &state->command);
    
    return state->cog;
}

void ws2812_close(ws2812_t *state)
{
    if (state->cog >= 0) {
        cogstop(state->cog);
        state->cog = -1;
    }
}

void ws2812_refresh(ws2812_t *state, int pin, uint32_t *colors, int count)
{
    uint32_t cmd;
    cmd =  pin
        | (count << 8)
        | ((uint32_t)colors << 16);
    while (state->command)
        ;
    state->command = cmd;
}
